@@ -27,6 +27,7 @@
 #  define BITMAP_BIT_ORDER LSBFirst
 #endif
 
+#undef DEBUG
 #define DEBUG 0
 
 #if defined (__GNUC__)
@@ -37,16 +38,49 @@
 #  define FUNC     ((const char*) ("???"))
 #endif
 
+#ifndef INT16_MIN
+# define INT16_MIN              (-32767-1)
+#endif
+
+#ifndef INT16_MAX
+# define INT16_MAX              (32767)
+#endif
+
+#ifndef INT32_MIN
+# define INT32_MIN              (-2147483647-1)
+#endif
+
+#ifndef INT32_MAX
+# define INT32_MAX              (2147483647)
+#endif
+
+#ifndef UINT32_MIN
+# define UINT32_MIN             (0)
+#endif
+
+#ifndef UINT32_MAX
+# define UINT32_MAX             (4294967295U)
+#endif
+
+#ifndef M_PI
+# define M_PI			3.14159265358979323846
+#endif
+
+#ifdef _MSC_VER
+#define inline __inline
+#endif
 
 #define FB_SHIFT    5
 #define FB_UNIT     (1 << FB_SHIFT)
 #define FB_HALFUNIT (1 << (FB_SHIFT-1))
 #define FB_MASK     (FB_UNIT - 1)
 #define FB_ALLONES  ((uint32_t) -1)
-    
+
 /* Memory allocation helpers */
 void *pixman_malloc_ab (unsigned int n, unsigned int b);
 void *pixman_malloc_abc (unsigned int a, unsigned int b, unsigned int c);
+pixman_bool_t pixman_multiply_overflows_int (unsigned int a, unsigned int b);
+pixman_bool_t pixman_addition_overflows_int (unsigned int a, unsigned int b);
 
 #if DEBUG
 
@@ -112,6 +146,12 @@ typedef struct point point_t;
 typedef FASTCALL void (*CombineMaskU) (uint32_t *src, const uint32_t *mask, int width);
 typedef FASTCALL void (*CombineFuncU) (uint32_t *dest, const uint32_t *src, int width);
 typedef FASTCALL void (*CombineFuncC) (uint32_t *dest, uint32_t *src, uint32_t *mask, int width);
+typedef FASTCALL void (*fetchProc)(bits_image_t *pict, int x, int y, int width,
+                                   uint32_t *buffer);
+typedef FASTCALL uint32_t (*fetchPixelProc)(bits_image_t *pict, int offset, int line);
+typedef FASTCALL void (*storeProc)(pixman_image_t *, uint32_t *bits,
+                                   const uint32_t *values, int x, int width,
+                                   const pixman_indexed_t *);
 
 typedef struct _FbComposeData {
     uint8_t	 op;
@@ -141,6 +181,32 @@ void pixman_composite_rect_general_accessors (const FbComposeData *data,
 void pixman_composite_rect_general (const FbComposeData *data,
 				    uint32_t *scanline_buffer);
 
+fetchProc pixman_fetchProcForPicture (bits_image_t *);
+fetchPixelProc pixman_fetchPixelProcForPicture (bits_image_t *);
+storeProc pixman_storeProcForPicture (bits_image_t *);
+fetchProc pixman_fetchProcForPicture_accessors (bits_image_t *);
+fetchPixelProc pixman_fetchPixelProcForPicture_accessors (bits_image_t *);
+storeProc pixman_storeProcForPicture_accessors (bits_image_t *);
+
+void pixmanFetchSourcePict(source_image_t *, int x, int y, int width,
+                           uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
+
+void fbFetchTransformed(bits_image_t *, int x, int y, int width,
+                        uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
+void fbStoreExternalAlpha(bits_image_t *, int x, int y, int width,
+                          uint32_t *buffer);
+void fbFetchExternalAlpha(bits_image_t *, int x, int y, int width,
+                          uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
+
+void fbFetchTransformed_accessors(bits_image_t *, int x, int y, int width,
+                                  uint32_t *buffer, uint32_t *mask,
+                                  uint32_t maskBits);
+void fbStoreExternalAlpha_accessors(bits_image_t *, int x, int y, int width,
+                                    uint32_t *buffer);
+void fbFetchExternalAlpha_accessors(bits_image_t *, int x, int y, int width,
+                                    uint32_t *buffer, uint32_t *mask,
+                                    uint32_t maskBits);
+
 /* end */
 
 typedef enum
@@ -197,7 +263,7 @@ struct solid_fill
     source_image_t	common;
     uint32_t		color;		/* FIXME: shouldn't this be a pixman_color_t? */
 };
-    
+
 struct gradient
 {
     source_image_t		common;
@@ -239,7 +305,7 @@ struct conical_gradient
     gradient_t			common;
     pixman_point_fixed_t	center;
     pixman_fixed_t		angle;
-}; 
+};
 
 struct bits_image
 {
@@ -265,6 +331,11 @@ union pixman_image
     solid_fill_t		solid;
 };
 
+
+extern CombineFuncU pixman_fbCombineFuncU[];
+extern CombineFuncC pixman_fbCombineFuncC[];
+FASTCALL void pixman_fbCombineMaskU (uint32_t *src, const uint32_t *mask, int width);
+
 #define LOG2_BITMAP_PAD 5
 #define FB_STIP_SHIFT	LOG2_BITMAP_PAD
 #define FB_STIP_UNIT	(1 << FB_STIP_SHIFT)
@@ -307,23 +378,23 @@ union pixman_image
     }
 
 #if IMAGE_BYTE_ORDER == MSBFirst
-#define Fetch24(a)  ((unsigned long) (a) & 1 ?			      \
-		     ((READ(a) << 16) | READ((uint16_t *) ((a)+1))) : \
-		     ((READ((uint16_t *) (a)) << 8) | READ((a)+2)))
-#define Store24(a,v) ((unsigned long) (a) & 1 ?		\
-		      (WRITE(a, (uint8_t) ((v) >> 16)),		      \
-		       WRITE((uint16_t *) ((a)+1), (uint16_t) (v))) :  \
-		      (WRITE((uint16_t *) (a), (uint16_t) ((v) >> 8)), \
-		       WRITE((a)+2, (uint8_t) (v))))
+#define Fetch24(img, a)  ((unsigned long) (a) & 1 ?	      \
+    ((READ(img, a) << 16) | READ(img, (uint16_t *) ((a)+1))) : \
+    ((READ(img, (uint16_t *) (a)) << 8) | READ(img, (a)+2)))
+#define Store24(img,a,v) ((unsigned long) (a) & 1 ? \
+    (WRITE(img, a, (uint8_t) ((v) >> 16)),		  \
+     WRITE(img, (uint16_t *) ((a)+1), (uint16_t) (v))) :  \
+    (WRITE(img, (uint16_t *) (a), (uint16_t) ((v) >> 8)), \
+     WRITE(img, (a)+2, (uint8_t) (v))))
 #else
-#define Fetch24(a)  ((unsigned long) (a) & 1 ?			     \
-		     (READ(a) | (READ((uint16_t *) ((a)+1)) << 8)) : \
-		     (READ((uint16_t *) (a)) | (READ((a)+2) << 16)))
-#define Store24(a,v) ((unsigned long) (a) & 1 ? \
-		      (WRITE(a, (uint8_t) (v)),				\
-		       WRITE((uint16_t *) ((a)+1), (uint16_t) ((v) >> 8))) : \
-		      (WRITE((uint16_t *) (a), (uint16_t) (v)),		\
-		       WRITE((a)+2, (uint8_t) ((v) >> 16))))
+#define Fetch24(img,a)  ((unsigned long) (a) & 1 ?			     \
+    (READ(img, a) | (READ(img, (uint16_t *) ((a)+1)) << 8)) : \
+    (READ(img, (uint16_t *) (a)) | (READ(img, (a)+2) << 16)))
+#define Store24(img,a,v) ((unsigned long) (a) & 1 ? \
+    (WRITE(img, a, (uint8_t) (v)),				\
+     WRITE(img, (uint16_t *) ((a)+1), (uint16_t) ((v) >> 8))) : \
+    (WRITE(img, (uint16_t *) (a), (uint16_t) (v)),		\
+     WRITE(img, (a)+2, (uint8_t) ((v) >> 16))))
 #endif
 
 #define Alpha(x) ((x) >> 24)
@@ -566,6 +637,8 @@ union pixman_image
 #define DIV(a,b) ((((a) < 0) == ((b) < 0)) ? (a) / (b) :		\
 		  ((a) - (b) + 1 - (((b) < 0) << 1)) / (b))
 
+#define CLIP(a,b,c) ((a) < (b) ? (b) : ((a) > (c) ? (c) : (a)))
+
 #if 0
 /* FIXME: the MOD macro above is equivalent, but faster I think */
 #define mod(a,b) ((b) == 1 ? 0 : (a) >= 0 ? (a) % (b) : (b) - (-a) % (b))
@@ -612,43 +685,38 @@ union pixman_image
 
 #ifdef PIXMAN_FB_ACCESSORS
 
-#define READ(ptr)							\
-    (image->common.read_func ((ptr), sizeof(*(ptr))))
-#define WRITE(ptr,val)							\
-    (image->common.write_func ((ptr), (val), sizeof (*(ptr))))
+#define READ(img, ptr)							\
+    ((img)->common.read_func ((ptr), sizeof(*(ptr))))
+#define WRITE(img, ptr,val)						\
+    ((img)->common.write_func ((ptr), (val), sizeof (*(ptr))))
 
-#define MEMCPY_WRAPPED(dst, src, size)					\
+#define MEMCPY_WRAPPED(img, dst, src, size)				\
     do {								\
 	size_t _i;							\
 	uint8_t *_dst = (uint8_t*)(dst), *_src = (uint8_t*)(src);	\
 	for(_i = 0; _i < size; _i++) {					\
-	    WRITE(_dst +_i, READ(_src + _i));				\
+	    WRITE((img), _dst +_i, READ((img), _src + _i));		\
 	}								\
     } while (0)
-	
-#define MEMSET_WRAPPED(dst, val, size)					\
+
+#define MEMSET_WRAPPED(img, dst, val, size)				\
     do {								\
 	size_t _i;							\
 	uint8_t *_dst = (uint8_t*)(dst);				\
-	for(_i = 0; _i < size; _i++) {					\
-	    WRITE(_dst +_i, (val));					\
+	for(_i = 0; _i < (size_t) size; _i++) {				\
+	    WRITE((img), _dst +_i, (val));				\
 	}								\
     } while (0)
 
-/* FIXME */
-#define fbPrepareAccess(x)
-#define fbFinishAccess(x)
-
 #else
 
-#define READ(ptr)		(*(ptr))
-#define WRITE(ptr, val)		(*(ptr) = (val))
-#define MEMCPY_WRAPPED(dst, src, size)					\
+#define READ(img, ptr)		(*(ptr))
+#define WRITE(img, ptr, val)	(*(ptr) = (val))
+#define MEMCPY_WRAPPED(img, dst, src, size)					\
     memcpy(dst, src, size)
-#define MEMSET_WRAPPED(dst, val, size)					\
+#define MEMSET_WRAPPED(img, dst, val, size)					\
     memset(dst, val, size)
-#define fbPrepareAccess(x)
-#define fbFinishAccess(x)
+
 #endif
 
 #define fbComposeGetSolid(img, res, fmt)				\
@@ -668,21 +736,21 @@ union pixman_image
 	    switch (PIXMAN_FORMAT_BPP((img)->bits.format))		\
 	    {								\
 	    case 32:							\
-		(res) = READ((uint32_t *)bits__);			\
+		(res) = READ(img, (uint32_t *)bits__);			\
 		break;							\
 	    case 24:							\
-		(res) = Fetch24 ((uint8_t *) bits__);			\
+		(res) = Fetch24(img, (uint8_t *) bits__);			\
 		break;							\
 	    case 16:							\
-		(res) = READ((uint16_t *) bits__);			\
+		(res) = READ(img, (uint16_t *) bits__);			\
 		(res) = cvt0565to0888(res);				\
 		break;							\
 	    case 8:							\
-		(res) = READ((uint8_t *) bits__);			\
+		(res) = READ(img, (uint8_t *) bits__);			\
 		(res) = (res) << 24;					\
 		break;							\
 	    case 1:							\
-		(res) = READ((uint32_t *) bits__);			\
+		(res) = READ(img, (uint32_t *) bits__);			\
 		(res) = FbLeftStipBits((res),1) ? 0xff000000 : 0x00000000; \
 		break;							\
 	    default:							\
@@ -712,7 +780,7 @@ union pixman_image
 	__bits__ = pict->bits.bits;					\
 	__stride__ = pict->bits.rowstride;				\
 	__bpp__ = PIXMAN_FORMAT_BPP(pict->bits.format);			\
-	(out_stride) = __stride__ * sizeof (uint32_t) / sizeof (type);	\
+	(out_stride) = __stride__ * (int) sizeof (uint32_t) / (int) sizeof (type);	\
 	(line) = ((type *) __bits__) +					\
 	    (out_stride) * (y) + (mul) * (x);				\
     } while (0)
@@ -724,7 +792,7 @@ union pixman_image
 
 #define MAX_ALPHA(n)	((1 << (n)) - 1)
 #define N_Y_FRAC(n)	((n) == 1 ? 1 : (1 << ((n)/2)) - 1)
-#define N_X_FRAC(n)	((1 << ((n)/2)) + 1)
+#define N_X_FRAC(n)	((n) == 1 ? 1 : (1 << ((n)/2)) + 1)
 
 #define STEP_Y_SMALL(n)	(pixman_fixed_1 / N_Y_FRAC(n))
 #define STEP_Y_BIG(n)	(pixman_fixed_1 - (N_Y_FRAC(n) - 1) * STEP_Y_SMALL(n))
@@ -774,6 +842,8 @@ pixman_rasterize_edges_accessors (pixman_image_t *image,
 				  pixman_fixed_t	b);
 
 
+#ifdef PIXMAN_TIMING
+
 /* Timing */
 static inline uint64_t
 oil_profile_stamp_rdtsc (void)
@@ -812,10 +882,11 @@ void pixman_timer_register (PixmanTimer *timer);
 									\
 	timer##tname.n_times++;						\
 	begin##tname = OIL_STAMP();
-	
+
 #define TIMER_END(tname)						\
         timer##tname.total += OIL_STAMP() - begin##tname;		\
     }
 
+#endif /* PIXMAN_TIMING */
 
 #endif /* PIXMAN_PRIVATE_H */
